@@ -13,7 +13,7 @@ use warnings;
 # restriction could be listed.
 use 5.010; 
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 use Carp;
 
@@ -104,7 +104,7 @@ sub _pack_leader
    }
 }
 
-sub _unpack_leader
+sub _peek_leader_type
 {
    my $self = shift;
 
@@ -112,26 +112,13 @@ sub _unpack_leader
       length $self->{record} or croak "Ran out of bytes before finding a leader";
 
       my ( $typenum ) = unpack( "C", $self->{record} );
+      my $type = $typenum >> 5;
+
+      return $type unless $type == DATA_META;
+
       substr( $self->{record}, 0, 1, "" );
 
-      my $type = $typenum >> 5;
       my $num  = $typenum & 0x1f;
-
-      if( $num == 0x1f ) {
-         ( $num ) = unpack( "C", $self->{record} );
-
-         if( $num < 0x80 ) {
-            substr( $self->{record}, 0, 1, "" );
-         }
-         else {
-            ( $num ) = unpack( "N", $self->{record} );
-            $num &= 0x7fffffff;
-            substr( $self->{record}, 0, 4, "" );
-         }
-      }
-
-      return $type, $num unless $type == DATA_META;
-
       if( $num == DATAMETA_CONSTRUCT ) {
          $self->unpackmeta_construct;
       }
@@ -144,6 +131,32 @@ sub _unpack_leader
    }
 }
 
+sub _unpack_leader
+{
+   my $self = shift;
+
+   my $type = $self->_peek_leader_type;
+   my ( $typenum ) = unpack( "C", $self->{record} );
+   substr( $self->{record}, 0, 1, "" );
+
+   my $num  = $typenum & 0x1f;
+
+   if( $num == 0x1f ) {
+      ( $num ) = unpack( "C", $self->{record} );
+
+      if( $num < 0x80 ) {
+         substr( $self->{record}, 0, 1, "" );
+      }
+      else {
+         ( $num ) = unpack( "N", $self->{record} );
+         $num &= 0x7fffffff;
+         substr( $self->{record}, 0, 4, "" );
+      }
+   }
+
+   return $type, $num;
+}
+
 sub pack_bool
 {
    my $self = shift;
@@ -155,7 +168,7 @@ sub pack_bool
 sub unpack_bool
 {
    my $self = shift;
-   my ( $type, $num ) = @_ ? @_ : $self->_unpack_leader();
+   my ( $type, $num ) = $self->_unpack_leader();
 
    $type == DATA_NUMBER or croak "Expected to unpack a number(bool) but did not find one";
    $num == DATANUM_BOOLFALSE and return 0;
@@ -218,7 +231,7 @@ sub pack_int
 sub unpack_int
 {
    my $self = shift;
-   my ( $type, $num ) = @_ ? @_ : $self->_unpack_leader();
+   my ( $type, $num ) = $self->_unpack_leader();
 
    $type == DATA_NUMBER or croak "Expected to unpack a number but did not find one";
    exists $pack_int_format{$num} or croak "Expected an integer subtype but got $num";
@@ -243,7 +256,7 @@ sub pack_str
 sub unpack_str
 {
    my $self = shift;
-   my ( $type, $num ) = @_ ? @_ : $self->_unpack_leader();
+   my ( $type, $num ) = $self->_unpack_leader();
 
    $type == DATA_STRING or croak "Expected to unpack a string but did not find one";
    length $self->{record} >= $num or croak "Can't pull $num bytes for string as there aren't enough";
@@ -251,6 +264,22 @@ sub unpack_str
    return decode_utf8( $octets );
 }
 
+# Temporary methods for null-terminated strings. Will be removed once we no
+# longer support protocol version 0
+sub pack_stringZ
+{
+   my $self = shift;
+   $self->{record} .= pack( "Z*", $_[0] );
+}
+
+sub unpack_stringZ
+{
+   my $self = shift;
+   my ( $string ) = unpack( "Z*", $self->{record} );
+   substr( $self->{record}, 0, 1 + length $string, "" );
+   return $string;
+}
+
 sub pack_obj
 {
    my $self = shift;
@@ -285,10 +314,11 @@ sub pack_obj
 sub unpack_obj
 {
    my $self = shift;
-   my ( $type, $num ) = @_ ? @_ : $self->_unpack_leader();
+   my ( $type, $num ) = $self->_unpack_leader();
 
    my $stream = $self->{stream};
 
+   $type == DATA_OBJECT or croak "Expected to unpack an object but did not find one";
    return undef unless $num;
    if( $num == 4 ) {
       my ( $id ) = unpack( "N", $self->{record} ); substr( $self->{record}, 0, 4, "" );
@@ -299,6 +329,81 @@ sub unpack_obj
    }
 }
 
+sub pack_list
+{
+   my $self = shift;
+   my ( $list, $listtype ) = @_;
+
+   ref $list eq "ARRAY" or croak "Cannot pack a list from non-ARRAY reference";
+   my $member_type = $listtype->member_type;
+
+   $self->_pack_leader( DATA_LIST, scalar @$list );
+   $self->pack_typed( $member_type, $_ ) for @$list;
+
+   return $self;
+}
+
+sub unpack_list
+{
+   my $self = shift;
+   my ( $listtype ) = @_;
+
+   my ( $type, $num ) = $self->_unpack_leader();
+   $type == DATA_LIST or croak "Expected to unpack a list but did not find one";
+
+   my $member_type = $listtype->member_type;
+   my @a;
+   foreach ( 1 .. $num ) {
+      push @a, $self->unpack_typed( $member_type );
+   }
+   return \@a;
+}
+
+sub pack_dict
+{
+   my $self = shift;
+   my ( $dict, $dicttype ) = @_;
+
+   ref $dict eq "HASH" or croak "Cannot pack a dict from non-HASH reference";
+   my $member_type = $dicttype->member_type;
+
+   my @keys = keys %$dict;
+   @keys = sort @keys if $SORT_HASH_KEYS;
+
+   $self->_pack_leader( DATA_DICT, scalar @keys );
+   if( $self->{stream}->_ver_tangence_strings ) {
+      $self->pack_str( $_ ) and $self->pack_typed( $member_type, $dict->{$_} ) for @keys;
+   }
+   else {
+      $self->pack_stringZ( $_ ) and $self->pack_typed( $member_type, $dict->{$_} ) for @keys;
+   }
+
+   return $self;
+}
+
+sub unpack_dict
+{
+   my $self = shift;
+   my ( $dicttype ) = @_;
+
+   my ( $type, $num ) = $self->_unpack_leader();
+   $type == DATA_DICT or croak "Expected to unpack a dict but did not find one";
+
+   my $member_type = $dicttype->member_type;
+   my %h;
+   foreach ( 1 .. $num ) {
+      my $key;
+      if( $self->{stream}->_ver_tangence_strings ) {
+         $key = $self->unpack_str();
+      }
+      else {
+         $key = $self->unpack_stringZ();
+      }
+      $h{$key} = $self->unpack_typed( $member_type );
+   }
+   return \%h;
+}
+
 sub packmeta_construct
 {
    my $self = shift;
@@ -314,7 +419,19 @@ sub packmeta_construct
    my $smashkeys = $class->smashkeys;
 
    $self->_pack_leader( DATA_META, DATAMETA_CONSTRUCT );
-   $self->{record} .= pack( "NZ*", $id, $class->perlname );
+   if( $stream->_ver_class_idnums ) {
+      $self->pack_int( $id );
+      $self->pack_int( $stream->peer_hasclass->{$class->perlname}->[2] );
+   }
+   else {
+      $self->{record} .= pack( "N", $id );
+      if( $stream->_ver_tangence_strings ) {
+         $self->pack_str( $class->perlname );
+      }
+      else {
+         $self->pack_stringZ( $class->perlname );
+      }
+   }
 
    my $smasharr = [];
 
@@ -341,7 +458,22 @@ sub unpackmeta_construct
 
    my $stream = $self->{stream};
 
-   my ( $id, $class ) = unpack( "NZ*", $self->{record} ); substr( $self->{record}, 0, 5 + length $class, "" );
+   my $id;
+   my $class;
+   if( $stream->_ver_class_idnums ) {
+      $id = $self->unpack_int();
+      my $classid = $self->unpack_int();
+      $class = $stream->message_state->{id2class}{$classid};
+   }
+   else {
+      ( $id ) = unpack( "N", $self->{record} ); substr( $self->{record}, 0, 4, "" );
+      if( $stream->_ver_tangence_strings ) {
+         $class = $self->unpack_str();
+      }
+      else {
+         $class = $self->unpack_stringZ();
+      }
+   }
    my $smasharr = $self->unpack_typed( TYPE_LIST_ANY );
 
    my $smashkeys = $stream->peer_hasclass->{$class}->[1];
@@ -361,7 +493,7 @@ sub packmeta_class
 
    $self->_pack_leader( DATA_META, DATAMETA_CLASS );
 
-   my $schema = {
+   my $introspection = {
       methods    => { 
          pairmap {
             $a => { args => [ map { $_->sig } $b->argtypes ], ret => ( $b->ret ? $b->ret->sig : "" ) }
@@ -384,12 +516,24 @@ sub packmeta_class
 
    my $smashkeys = $class->smashkeys;
 
-   # TODO: This ought to be totally redone sometime
-   $self->{record} .= pack( "Z*", $class->perlname );
-   $self->pack_typed( TYPE_DICT_ANY, $schema );
+   my $classid;
+   if( $stream->_ver_class_idnums ) {
+      $classid = ++$stream->message_state->{next_classid};
+      $self->pack_str( $class->perlname );
+      $self->pack_int( $classid );
+   }
+   else {
+      if( $stream->_ver_tangence_strings ) {
+         $self->pack_str( $class->perlname );
+      }
+      else {
+         $self->pack_stringZ( $self->perlname );
+      }
+   }
+   $self->pack_typed( TYPE_DICT_ANY, $introspection );
    $self->pack_typed( TYPE_LIST_STR, $smashkeys );
 
-   $stream->peer_hasclass->{$class->perlname} = [ $schema, $smashkeys ];
+   $stream->peer_hasclass->{$class->perlname} = [ $introspection, $smashkeys, $classid ];
 }
 
 sub unpackmeta_class
@@ -398,22 +542,87 @@ sub unpackmeta_class
 
    my $stream = $self->{stream};
 
-   my ( $class ) = unpack( "Z*", $self->{record} ); substr( $self->{record}, 0, 1 + length $class, "" );
-   my $schema    = $self->unpack_typed( TYPE_DICT_ANY );
-   my $smashkeys = $self->unpack_typed( TYPE_LIST_STR );
+   my $class;
+   my $classid;
+   if( $stream->_ver_class_idnums ) {
+      $class = $self->unpack_str();
+      $classid = $self->unpack_int();
+   }
+   else {
+      if( $stream->_ver_tangence_strings ) {
+         $class = $self->unpack_str();
+      }
+      else {
+         $class = $self->unpack_stringZ();
+      }
+   }
+   my $introspection = $self->unpack_typed( TYPE_DICT_ANY );
+   my $smashkeys     = $self->unpack_typed( TYPE_LIST_STR );
 
-   foreach my $mdef ( values %{ $schema->{methods} } ) {
+   foreach my $mdef ( values %{ $introspection->{methods} } ) {
       $_ = Tangence::Meta::Type->new_from_sig( $_ ) for @{ $mdef->{args} };
       length and $_ = Tangence::Meta::Type->new_from_sig( $_) for $mdef->{ret};
    }
-   foreach my $edef ( values %{ $schema->{events} } ) {
+   foreach my $edef ( values %{ $introspection->{events} } ) {
       $_ = Tangence::Meta::Type->new_from_sig( $_ ) for @{ $edef->{args} };
    }
-   foreach my $pdef ( values %{ $schema->{properties} } ) {
+   foreach my $pdef ( values %{ $introspection->{properties} } ) {
       $_ = Tangence::Meta::Type->new_from_sig( $_ ) for $pdef->{type};
    }
 
-   $stream->peer_hasclass->{$class} = [ $schema, $smashkeys ];
+   $stream->peer_hasclass->{$class} = [ $introspection, $smashkeys, $classid ];
+   if( defined $classid ) {
+      $stream->message_state->{id2class}{$classid} = $class;
+   }
+}
+
+# Used by pack_typed
+sub pack_prim
+{
+   my $self = shift;
+   my ( $d, $type ) = @_;
+
+   my $sig = $type->sig;
+
+   if( my $code = $self->can( "pack_$sig" ) ) {
+      $code->( $self, $d );
+   }
+   elsif( exists $int_sigs{$sig} ) {
+      ref $d and croak "$d is not a number";
+      my $subtype = $int_sigs{$sig};
+      $self->_pack_leader( DATA_NUMBER, $subtype );
+      $self->{record} .= pack( $pack_int_format{$subtype}[0], $d );
+   }
+   else {
+      croak "Unrecognised type signature $sig";
+   }
+
+   return $self;
+}
+
+# Used by unpack_typed
+sub unpack_prim
+{
+   my $self = shift;
+   my ( $type ) = @_;
+
+   my $sig = $type->sig;
+
+   if( my $code = $self->can( "unpack_$sig" ) ) {
+      return $code->( $self );
+   }
+   elsif( exists $int_sigs{$sig} ) {
+      my ( $type, $num ) = $self->_unpack_leader();
+
+      $type == DATA_NUMBER or croak "Expected to unpack a number but did not find one";
+      $num == $int_sigs{$sig} or croak "Expected subtype $int_sigs{$sig} but got $num";
+      my ( $n ) = unpack( $pack_int_format{$num}[0], $self->{record} );
+      substr( $self->{record}, 0, $pack_int_format{$num}[1] ) = "";
+      return $n;
+   }
+   else {
+      croak "Unrecognised type signature $sig";
+   }
 }
 
 sub pack_any
@@ -421,6 +630,8 @@ sub pack_any
    my $self = shift;
    my ( $d ) = @_;
 
+   my $stream = $self->{stream};
+
    if( !defined $d ) {
       $self->pack_obj( undef );
    }
@@ -432,14 +643,10 @@ sub pack_any
       $self->pack_obj( $d );
    }
    elsif( ref $d eq "ARRAY" ) {
-      $self->_pack_leader( DATA_LIST, scalar @$d );
-      $self->pack_any( $_ ) for @$d;
+      $self->pack_list( $d, TYPE_LIST_ANY );
    }
    elsif( ref $d eq "HASH" ) {
-      my @keys = keys %$d;
-      @keys = sort @keys if $SORT_HASH_KEYS;
-      $self->_pack_leader( DATA_DICT, scalar @keys );
-      $self->{record} .= pack( "Z*", $_ ) and $self->pack_any( $d->{$_} ) for @keys;
+      $self->pack_dict( $d, TYPE_DICT_ANY );
    }
    else {
       croak "Do not know how to pack a " . ref($d);
@@ -452,31 +659,22 @@ sub unpack_any
 {
    my $self = shift;
 
-   my ( $type, $num ) = $self->_unpack_leader();
+   my $type = $self->_peek_leader_type();
 
    if( $type == DATA_NUMBER ) {
-      return $self->unpack_int( $type, $num );
+      return $self->unpack_int();
    }
    if( $type == DATA_STRING ) {
-      return $self->unpack_str( $type, $num );
+      return $self->unpack_str();
    }
    elsif( $type == DATA_OBJECT ) {
-      return $self->unpack_obj( $type, $num );
+      return $self->unpack_obj();
    }
    elsif( $type == DATA_LIST ) {
-      my @a;
-      foreach ( 1 .. $num ) {
-         push @a, $self->unpack_any();
-      }
-      return \@a;
+      return $self->unpack_list( TYPE_LIST_ANY );
    }
    elsif( $type == DATA_DICT ) {
-      my %h;
-      foreach ( 1 .. $num ) {
-         my ( $key ) = unpack( "Z*", $self->{record} ); substr( $self->{record}, 0, 1 + length $key, "" );
-         $h{$key} = $self->unpack_any();
-      }
-      return \%h;
+      return $self->unpack_dict( TYPE_DICT_ANY );
    }
    else {
       croak "Do not know how to unpack record of type $type";
@@ -488,41 +686,8 @@ sub pack_typed
    my $self = shift;
    my ( $type, $d ) = @_;
 
-   if( $type->aggregate eq "prim" ) {
-      my $sig = $type->sig;
-
-      if( my $code = $self->can( "pack_$sig" ) ) {
-         $code->( $self, $d );
-      }
-      elsif( exists $int_sigs{$sig} ) {
-         ref $d and croak "$d is not a number";
-         my $subtype = $int_sigs{$sig};
-         $self->_pack_leader( DATA_NUMBER, $subtype );
-         $self->{record} .= pack( $pack_int_format{$subtype}[0], $d );
-      }
-      else {
-         croak "Unrecognised type signature $sig";
-      }
-   }
-   elsif( $type->aggregate eq "list" ) {
-      my $subtype = $type->member_type;
-      ref $d eq "ARRAY" or croak "Cannot pack a list from non-ARRAY reference";
-      $self->_pack_leader( DATA_LIST, scalar @$d );
-      $self->pack_typed( $subtype, $_ ) for @$d;
-   }
-   elsif( $type->aggregate eq "dict" ) {
-      my $subtype = $type->member_type;
-      ref $d eq "HASH" or croak "Cannot pack a dict from non-HASH reference";
-      my @keys = keys %$d;
-      @keys = sort @keys if $SORT_HASH_KEYS;
-      $self->_pack_leader( DATA_DICT, scalar @keys );
-      $self->{record} .= pack( "Z*", $_ ) and $self->pack_typed( $subtype, $d->{$_} ) for @keys;
-   }
-   else {
-      croak "Unrecognised type aggregation ".$type->aggregate;
-   }
-
-   return $self;
+   my $code = $self->can( "pack_" . $type->aggregate ) or die "Unrecognised type aggregation " . $type->aggregate;
+   return $code->( $self, $d, $type );
 }
 
 sub unpack_typed
@@ -530,51 +695,8 @@ sub unpack_typed
    my $self = shift;
    my $type = shift;
 
-   if( $type->aggregate eq "prim" ) {
-      my $sig = $type->sig;
-
-      if( my $code = $self->can( "unpack_$sig" ) ) {
-         return $code->( $self );
-      }
-      elsif( exists $int_sigs{$sig} ) {
-         my ( $type, $num ) = $self->_unpack_leader();
-
-         $type == DATA_NUMBER or croak "Expected to unpack a number but did not find one";
-         $num == $int_sigs{$sig} or croak "Expected subtype $int_sigs{$sig} but got $num";
-         my ( $n ) = unpack( $pack_int_format{$num}[0], $self->{record} );
-         substr( $self->{record}, 0, $pack_int_format{$num}[1] ) = "";
-         return $n;
-      }
-      else {
-         croak "Unrecognised type signature $sig";
-      }
-   }
-   elsif( $type->aggregate eq "list" ) {
-      my $subtype = $type->member_type;
-      my ( $type, $num ) = $self->_unpack_leader();
-
-      $type == DATA_LIST or croak "Expected to unpack a list but did not find one";
-      my @a;
-      foreach ( 1 .. $num ) {
-         push @a, $self->unpack_typed( $subtype );
-      }
-      return \@a;
-   }
-   elsif( $type->aggregate eq "dict" ) {
-      my $subtype = $type->member_type;
-      my ( $type, $num ) = $self->_unpack_leader();
-
-      $type == DATA_DICT or croak "Expected to unpack a dict but did not find one";
-      my %h;
-      foreach ( 1 .. $num ) {
-         my ( $key ) = unpack( "Z*", $self->{record} ); substr( $self->{record}, 0, 1 + length $key, "" );
-         $h{$key} = $self->unpack_typed( $subtype );
-      }
-      return \%h;
-   }
-   else {
-      croak "Unrecognised type aggregation ".$type->aggregate;
-   }
+   my $code = $self->can( "unpack_" . $type->aggregate ) or die "Unrecognised type aggregation " . $type->aggregate;
+   return $code->( $self, $type );
 }
 
 sub pack_all_typed
